# episoder, https://code.ott.net/episoder
#
# Copyright (C) 2004-2020 Stefan Ott. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from sys import stdout
from datetime import date, timedelta
from smtplib import SMTP


class FormattingRenderer(object):

	def __init__(self, fmt, dateformat):

		self._format = fmt
		self._dateformat = dateformat

	def format(self, episode):

		string = self._format

		airdate = episode.airdate.strftime(self._dateformat)
		string = string.replace("%airdate", airdate)
		string = string.replace("%show", episode.show.name)
		string = string.replace("%season", str(episode.season))
		string = string.replace("%epnum", "%02d" % episode.episode)
		string = string.replace("%eptitle", str(episode.title))
		string = string.replace("%totalep", str(episode.totalnum))
		string = string.replace("%prodnum", episode.prodnum)

		return string


class ConsoleRenderer(FormattingRenderer):

	RED = "\033[31;1m"
	CYAN = "\033[36;1m"
	GREY = "\033[30;0m"
	GREEN = "\033[32;1m"
	YELLOW = "\033[33;1m"

	def __init__(self, fmt, dateformat, dest=stdout):

		super(ConsoleRenderer, self).__init__(fmt, dateformat)
		self._dest = dest

	def __str__(self):

		return "ConsoleRenderer"

	def __repr__(self):

		return 'ConsoleRenderer("%s", "%s", <..>)' % (self._format,
							self._dateformat)

	def _render_color(self, text, col):

		self._dest.write("%s%s%s\n" % (col, text, ConsoleRenderer.GREY))

	def _render_no_color(self, string, _):

		self._dest.write("%s\n" % (string))

	def _render(self, episode, render, color):

		render(self.format(episode), color)

	def render(self, episodes, use_colors=True):

		today = date.today()
		yesterday = today - timedelta(1)
		tomorrow = today + timedelta(1)

		if use_colors:
			render = self._render_color
		else:
			render = self._render_no_color

		for episode in episodes:

			if episode.airdate == yesterday:
				color = ConsoleRenderer.RED
			elif episode.airdate == today:
				color = ConsoleRenderer.YELLOW
			elif episode.airdate == tomorrow:
				color = ConsoleRenderer.GREEN
			elif episode.airdate > tomorrow:
				color = ConsoleRenderer.CYAN
			else:
				color = ConsoleRenderer.GREY

			self._render(episode, render, color)


class NewEpisodesNotification(FormattingRenderer):

	def __init__(self, episodes, fmt, datefmt):

		super(NewEpisodesNotification, self).__init__(fmt, datefmt)
		self._episodes = episodes

	def __str__(self):

		return "Notification: %s new episode(s)" % len(self._episodes)

	def __repr__(self):

		eps = "<(%d x Episode)" % len(self._episodes)
		fmt = 'NewEpisodesNotification(%s, "%s", "%s")'
		return fmt % (eps, self._format, self._dateformat)

	def get(self):

		return "".join("%s\n" % self.format(e) for e in self._episodes)

	def send(self, notifier, to, pretend=False):

		body = "Your upcoming episodes:\n\n"

		for episode in self._episodes:
			body += "%s\n" % self.format(episode)

			if not pretend:
				episode.notified = date.today()

		notifier.send(body, to, pretend)


class EmailNotifier(object):

	def __init__(self, server, port, smtp=SMTP):

		self._server = server
		self._port = port
		self._smtp = smtp
		self._user = None
		self._password = None

		self.use_tls = False

	def __str__(self):

		return "EmailNotifier"

	def __repr__(self):

		fmt = 'EmailNotifier("%s", %d, %s)'
		return fmt % (self._server, self._port, str(self._smtp))

	def _get_header(self, to, subject):

		return "From: %s\nTo: %s\nSubject: %s" % (to, to, subject)

	def login(self, user, password):

		self._user = user
		self._password = password

	def send(self, body, to, pretend=False):

		header = self._get_header(to, "Upcoming TV episodes")
		message = "%s\n%s" % (header, body)

		if pretend:
			print(message)
			return

		server = self._smtp(self._server, self._port)

		if self.use_tls:
			server.starttls()
		if self._user:
			server.login(self._user, self._password)

		server.sendmail(to, to, message)
		server.quit()



# episoder, https://code.ott.net/episoder
#
# Copyright (C) 2004-2020 Stefan Ott. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import logging
from datetime import date, timedelta

import sqlite3
from sqlalchemy import Table, MetaData, create_engine, or_, and_
from sqlalchemy.orm import create_session

from .database import Episode, Show, Meta


class Database(object):

	def __init__(self, path):

		self._path = path
		self.logger = logging.getLogger("Database")

		self.open()
		self._initdb()

	def __str__(self):

		return "Episoder Database at %s" % self._path

	def __repr__(self):

		return "Database(%s)" % self._path

	def _initdb(self):

		# Initialize the database if all tables are missing
		tables = [Show, Episode, Meta]
		tables = map(lambda x: x.__table__.exists, tables)
		found = [x for x in tables if x(bind=self.engine)]

		if len(found) < 1:
			Show.__table__.create(bind=self.engine)
			Episode.__table__.create(bind=self.engine)
			Meta.__table__.create(bind=self.engine)
			self.set_schema_version(4)

	def open(self):

		if self._path.find("://") > -1:
			self.engine = create_engine(self._path,
							convert_unicode=True)
		else:
			self.engine = create_engine("sqlite:///%s" % self._path)

		self.conn = self.engine.connect()
		self.metadata = MetaData()
		self.metadata.bind = self.engine
		self.session = create_session(bind=self.engine)
		self.session.begin()

	def close(self):

		self.session.commit()
		self.session.close()
		self.conn.close()
		self.engine.dispose()

	def set_schema_version(self, version):

		meta = Meta()
		meta.key = "schema"
		meta.value = "%d" % version

		self.session.merge(meta)
		self.session.flush()

	def get_schema_version(self):

		if not Meta.__table__.exists(bind=self.engine):
			return 1

		res = self.session.query(Meta).get("schema")
		if res:
			return int(res.value)

		return 0

	def clear(self):

		episodes = self.session.query(Episode).all()

		for episode in episodes:
			self.session.delete(episode)

		self.session.flush()

	def migrate(self):

		schema_version = self.get_schema_version()
		self.logger.debug("Found schema version %s", schema_version)

		if schema_version < 0:

			self.logger.debug("Automatic schema updates disabled")
			return

		if schema_version == 1:

			# Upgrades from version 1 are rather harsh, we
			# simply drop and re-create the tables
			self.logger.debug("Upgrading to schema version 2")

			table = Table("episodes", self.metadata, autoload=True)
			table.drop()

			table = Table("shows", self.metadata, autoload=True)
			table.drop()

			Show.__table__.create(bind=self.engine)
			Episode.__table__.create(bind=self.engine)
			Meta.__table__.create(bind=self.engine)

			schema_version = 4
			self.set_schema_version(schema_version)

		if schema_version == 2:

			# Add two new columns to the shows table
			self.logger.debug("Upgrading to schema version 3")

			# We can only do this with sqlite databases
			assert self.engine.driver == "pysqlite"

			self.close()

			upgrade = sqlite3.connect(self._path)
			upgrade.execute("ALTER TABLE shows "
					"ADD COLUMN enabled TYPE boolean")
			upgrade.execute("ALTER TABLE shows "
					"ADD COLUMN status TYPE integer")
			upgrade.close()

			self.open()
			schema_version = 3
			self.set_schema_version(schema_version)

		if schema_version == 3:

			# Add a new column to the episodes table
			self.logger.debug("Upgrading to schema version 4")

			# We can only do this with sqlite databases
			assert self.engine.driver == "pysqlite"

			self.close()

			upgrade = sqlite3.connect(self._path)
			upgrade.execute("ALTER TABLE episodes "
					"ADD COLUMN notified TYPE date")
			upgrade.close()

			self.open()
			schema_version = 4
			self.set_schema_version(schema_version)

	def get_expired_shows(self, today=date.today()):

		delta_running = timedelta(2)	# 2 days
		delta_suspended = timedelta(7)	# 1 week
		delta_ended = timedelta(14)	# 2 weeks

		shows = self.session.query(Show).filter(or_(
				and_(
					Show.enabled,
					Show.status == Show.RUNNING,
					Show.updated < today - delta_running
				),
				and_(
					Show.enabled,
					Show.status == Show.SUSPENDED,
					Show.updated < today - delta_suspended
				),
				and_(
					Show.enabled,
					Show.status == Show.ENDED,
					Show.updated < today - delta_ended
				)
		))

		return shows.all()

	def get_enabled_shows(self):

		shows = self.session.query(Show).filter(Show.enabled)
		return shows.all()

	def get_show_by_url(self, url):

		shows = self.session.query(Show).filter(Show.url == url)

		if shows.count() < 1:
			return None

		return shows.first()

	def get_show_by_id(self, show_id):

		return self.session.query(Show).get(show_id)

	def add_show(self, show):

		show = self.session.merge(show)
		self.session.flush()
		return show

	def remove_show(self, show_id):

		show = self.session.query(Show).get(show_id)

		if not show:
			self.logger.error("No such show")
			return

		episodes = self.session.query(Episode)

		for episode in episodes.filter(Episode.show_id == show.id):
			self.session.delete(episode)

		self.session.delete(show)
		self.session.flush()

	def get_shows(self):

		return self.session.query(Show).all()

	def add_episode(self, episode, show):

		episode.show_id = show.id
		self.session.merge(episode)
		self.session.flush()

	def get_episodes(self, basedate=date.today(), n_days=0):

		enddate = basedate + timedelta(n_days)

		return self.session.query(Episode).\
			filter(Episode.airdate >= basedate). \
			filter(Episode.airdate <= enddate). \
			order_by(Episode.airdate).all()

	def search(self, search):

		return self.session.query(Episode).\
			filter(or_( \
				Episode.title.like("%%%s%%" % search),
				Show.name.like("%%%s%%" % search))). \
			order_by(Episode.airdate).all()

	def commit(self):

		self.session.commit()
		self.session.begin()

	def rollback(self):

		self.session.rollback()
		self.session.begin()

	def remove_before(self, then, show=None):

		eps = self.session.query(Episode).filter(Episode.airdate < then)

		if show:
			eps = eps.filter(Episode.show == show)

		for episode in eps:
			self.session.delete(episode)

		self.commit()



# episoder, https://code.ott.net/episoder
#
# Copyright (C) 2004-2020 Stefan Ott. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from datetime import datetime

from sqlalchemy import Column, Integer, Text, Date, ForeignKey, Sequence
from sqlalchemy import DateTime, Boolean
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref

Base = declarative_base()


class Show(Base):

	RUNNING = 1
	SUSPENDED = 2
	ENDED = 3

	__tablename__ = "shows"

	id = Column("show_id", Integer, Sequence("shows_show_id_seq"),
							primary_key=True)
	name = Column("show_name", Text)
	url = Column(Text, unique=True)
	updated = Column(DateTime)
	enabled = Column(Boolean)
	status = Column(Integer, default=RUNNING)

	def __init__(self, name, url=u"", updated=datetime.fromtimestamp(0)):

		self.name = name
		self.url = url
		self.updated = updated
		self.status = Show.RUNNING
		self.episodes = []
		self.enabled = True

	def __str__(self):

		return "Show: %s" % self.name

	def __repr__(self):

		return 'Show(u"%s", u"%s")' % (self.name, self.url)

	def __eq__(self, other):

		return self.name == other.name and self.url == other.url

	def remove_episodes_before(self, store, date):

		logging.debug("Removing episodes from before %s", date)
		store.remove_before(date, show=self)


class Episode(Base):

	__tablename__ = "episodes"

	show_id = Column(Integer, ForeignKey(Show.id), primary_key=True)
	episode = Column("num", Integer, primary_key=True)
	airdate = Column(Date)
	season = Column(Integer, primary_key=True)
	title = Column(Text)
	totalnum = Column(Integer)
	prodnum = Column(Text)
	notified = Column(Date)
	show = relationship(Show, backref=backref("episodes", cascade="all"))

	def __init__(self, name, season, episode, aired, prodnum, total):

		self.title = name
		self.season = int(season)
		self.episode = int(episode)
		self.airdate = aired
		self.prodnum = prodnum
		self.totalnum = int(total)

	def __lt__(self, other):

		return ((self.season < other.season)
			or ((self.season == other.season)
				and (self.episode < other.episode)))

	def __str__(self):

		return "%s %dx%02d: %s" % (self.show.name, self.season,
			self.episode, self.title)

	def __repr__(self):

		return 'Episode(u"%s", %d, %d, date(%d, %d, %d), u"%s", %d)' % (
				self.title, self.season, self.episode,
				self.airdate.year, self.airdate.month,
				self.airdate.day, self.prodnum, self.totalnum)

	def __eq__(self, other):

		return (self.show_id == other.show_id
			and self.season == other.season
			and self.episode == other.episode)


class Meta(Base):

	__tablename__ = "meta"
	key = Column(Text, primary_key=True)
	value = Column(Text)

	def __str__(self):

		return "Meta: %s = %s" % (self.key, self.value)

	def __repr__(self):

		return str(self)



# episoder, https://code.ott.net/episoder
#
# Copyright (C) 2004-2020 Stefan Ott. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from re import search, match
from json import dumps
from datetime import datetime

import requests

from .database import Episode, Show


class ParserSelector(object):

	class _Inner(object):

		def __init__(self):

			self._tvdb = TVDB()
			self._epguides = Epguides()
			self._tvcom = TVCom()

		def parser_for(self, url):

			for parser in [self._tvdb, self._epguides, self._tvcom]:
				if parser.accept(url):
					return parser

			return None

	_instance = None

	@staticmethod
	def instance():

		if not ParserSelector._instance:
			ParserSelector._instance = ParserSelector._Inner()

		return ParserSelector._instance


class InvalidLoginError(Exception):

	pass


class TVDBShowNotFoundError(Exception):

	pass


class TVDBNotLoggedInError(Exception):

	pass


class TVDBOffline(object):

	def __init__(self, tvdb):

		self._tvdb = tvdb
		self._log = logging.getLogger("TVDB (offline)")

	def __str__(self):

		return "thetvdb.com parser (ready)"

	def __repr__(self):

		return "<TVDBOffline>"

	def _post_login(self, data, user_agent):

		url = "https://api.thetvdb.com/login"
		head = {"Content-type": "application/json",
			"User-Agent": user_agent}
		body = dumps(data).encode("utf8")
		response = requests.post(url, body, headers=head)
		data = response.json()

		if response.status_code == 401:
			raise InvalidLoginError(data.get("Error"))

		self._log.info("Successful login")
		return data.get("token")

	def lookup(self, _, __):

		raise TVDBNotLoggedInError()

	def login(self, args):

		body = {"apikey": args.tvdb_key}
		self.token = self._post_login(body, args.agent)

	def parse(self, _, __, ___):

		raise TVDBNotLoggedInError()

	def _set_token(self, token):

		self._tvdb.change(TVDBOnline(token))

	token = property(None, _set_token)


class TVDBOnline(object):

	def __init__(self, token):

		self._token = token
		self._log = logging.getLogger("TVDB (online)")

	def __str__(self):

		return "thetvdb.com parser (authorized)"

	def __repr__(self):

		return "<TVDBOnline>"

	def _get(self, url, params, agent):

		url = "https://api.thetvdb.com/%s" % url
		head = {"Content-type": "application/json",
			"User-Agent": agent,
			"Authorization": "Bearer %s" % self._token}
		response = requests.get(url, headers=head, params=params)
		data = response.json()

		if response.status_code == 404:
			raise TVDBShowNotFoundError(data.get("Error"))

		return data

	def _get_episodes(self, show, page, agent):

		show_id = int(show.url)
		opts = {"page": page}
		result = self._get("series/%d/episodes" % show_id, opts, agent)
		return (result.get("data"), result.get("links"))

	def lookup(self, term, agent):

		def mkshow(entry):

			name = entry.get("seriesName")
			url = str(entry.get("id")).encode("utf8").decode("utf8")
			return Show(name, url=url)

		matches = self._get("search/series", {"name": term}, agent)
		return map(mkshow, matches.get("data"))

	def login(self, args):

		pass

	def _fetch_episodes(self, show, page, agent):

		def airdate(row):

			aired = row.get("firstAired")

			if aired == "0000-00-00":
				return datetime(1970, 1, 1)

			return datetime.strptime(aired, "%Y-%m-%d").date()

		def mkepisode(row):

			num = int(row.get("airedEpisodeNumber", "0") or 0)
			name = row.get("episodeName") or u"Unnamed episode"
			season = int(row.get("airedSeason", "0") or 0)
			aired = airdate(row)
			pnum = u"UNK"

			self._log.debug("Found episode %s", name)
			return Episode(name, season, num, aired, pnum, 0)

		def isvalid(row):

			return row.get("firstAired") not in [None, ""]

		(data, links) = self._get_episodes(show, page, agent)
		valid = filter(isvalid, data)
		episodes = [mkepisode(row) for row in valid]

		# handle pagination
		next_ = links.get("next") or 0
		if next_ > page:
			try:
				more = self._fetch_episodes(show, next_, agent)
				episodes.extend(more)
			except TVDBShowNotFoundError:
				msg = "Error parsing %s: failed to load page %d"
				self._log.error(msg, show.name, next_)

		return episodes

	def parse(self, show, db, user_agent):

		result = self._get("series/%d" % int(show.url), {}, user_agent)
		data = result.get("data")

		# update show data
		show.name = data.get("seriesName", show.name)
		show.updated = datetime.now()

		self._log.debug("Updating show '%s'", show.name)

		if data.get("status") == "Continuing":
			show.status = Show.RUNNING
		else:
			show.status = Show.ENDED

		# load episodes
		try:
			episodes = self._fetch_episodes(show, 1, user_agent)
		except TVDBShowNotFoundError:
			#raise
			return

		for (idx, episode) in enumerate(sorted(episodes)):

			episode.totalnum = idx + 1
			db.add_episode(episode, show)

		db.commit()


class TVDB(object):

	def __init__(self):

		self._state = TVDBOffline(self)

	def __str__(self):

		return str(self._state)

	def __repr__(self):

		return "TVDB %s" % repr(self._state)

	def login(self, args):

		self._state.login(args)

	def lookup(self, text, args):

		return self._state.lookup(text, args.agent)

	def parse(self, show, db, args):

		return self._state.parse(show, db, args.agent)

	def change(self, state):

		self._state = state

	@staticmethod
	def accept(url):

		return url.isdigit()


class Epguides(object):

	def __init__(self):

		self.logger = logging.getLogger("Epguides")

	def __str__(self):

		return "epguides.com parser"

	def __repr__(self):

		return "Epguides()"

	@staticmethod
	def accept(url):

		return "epguides.com/" in url

	def login(self, args):

		pass

	def guess_encoding(self, response):

		raw = response.raw.read()
		text = raw.decode("iso-8859-1")

		if "charset=iso-8859-1" in text:
			return "iso-8859-1"

		return "utf8"

	def parse(self, show, db, args):

		headers = {"User-Agent": args.agent}
		response = requests.get(show.url, headers=headers)
		response.encoding = self.guess_encoding(response)

		for line in response.text.split("\n"):
			self._parse_line(line, show, db)

		show.updated = datetime.now()
		db.commit()

	def _parse_line(self, line, show, db):

		# Name of the show
		res = search(r"<title>(.*)</title>", line)
		if res:
			title = res.groups()[0]
			show.name = title.split(" (a ")[0]

		# Current status (running / ended)
		res = search(r'<span class="status">(.*)</span>', line)
		if res:
			text = res.groups()[0]
			if "current" in text:
				show.status = Show.RUNNING
			else:
				show.status = Show.ENDED
		else:
			if search(r"aired.*to.*[\d+]", line):
				show.status = Show.ENDED

		# Known formatting supported by this fine regex:
		# 4.     1-4            19 Jun 02  <a [..]>title</a>
		#   1.  19- 1   01-01    5 Jan 88  <a [..]>title</a>
		# 23     3-05           27/Mar/98  <a [..]>title</a>
		# 65.   17-10           23 Apr 05  <a [..]>title</a>
		# 101.   5-15           09 May 09  <a [..]>title</a>
		# 254.    - 5  05-254   15 Jan 92  <a [..]>title</a>
		res = search(r"^ *(\d+)\.? +(\d*)- ?(\d+) +([a-zA-Z0-9-]*)"\
		r" +(\d{1,2}[ /][A-Z][a-z]{2}[ /]\d{2}) *<a.*>(.*)</a>", line)

		if res:
			fields = res.groups()
			(total, season, epnum, prodnum, day, title) = fields

			day = day.replace("/", " ")
			airtime = datetime.strptime(day, "%d %b %y")

			self.logger.debug("Found episode %s", title)
			db.add_episode(Episode(title, season or 0, epnum,
					airtime.date(), prodnum, total), show)


class TVCom(object):

	def __init__(self):

		self._log = logging.getLogger("TVCom")

	def __str__(self):

		return "dummy tv.com parser to detect old urls"

	def __repr__(self):

		return "TVCom()"

	@staticmethod
	def accept(url):

		exp = "http://(www.)?tv.com/.*"
		return match(exp, url)

	def parse(self, source, _, __):

		self._log.error("The url %s is no longer supported", source.url)

	def login(self):

		pass



#!/usr/bin/env python
from os import path
from setuptools import setup

# fallback
__version__ = "0.0.0"

# files to install
files = []

# man page
manpage = "episoder.1"
files.append((path.join("man", "man1"), ["episoder.1"]))

# documentation
for file in ["AUTHORS", "CHANGELOG", "COPYING", "README.txt"]:
	files.append((path.join("doc", "episoder"), [file]))

# get version number
exec(open("pyepisoder/version.py").read())

files.append((path.join("doc", "episoder", "examples"),
						["examples/home.episoder"]))

setup(	name			= "episoder",
	version			= __version__,
	license			= "GPLv3",
	description		= "TV episode notifier",
	author			= "Stefan Ott",
	author_email		= "stefan@ott.net",
	url			= "https://code.ott.net/episoder",
	packages		= ["pyepisoder"],
	test_suite		= "test",
	scripts			= ["episoder"],
	long_description	= "episoder is a tool to tell you about new episodes of your favourite TV shows",
	data_files		= files,
	install_requires	= ["argparse", "requests", "sqlalchemy>=0.7"],
	classifiers		= [
		"Development Status :: 5 - Production/Stable",
		"Environment :: Console",
		"Intended Audience :: End Users/Desktop",
		"License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
		"Natural Language :: English",
		"Operating System :: OS Independent",
		"Programming Language :: Python",
		"Programming Language :: Python :: 2.6",
		"Programming Language :: Python :: 2.7",
		"Programming Language :: Python :: 3",
		"Topic :: Multimedia :: Video"
	]
)
